@@ -15,15 +15,16 @@
  * @category   Zend
  * @package    Zend_Cache
  * @subpackage Zend_Cache_Backend
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Sqlite.php 24593 2012-01-05 20:35:02Z matthew $
  */
 
 
 /**
  * @see Zend_Cache_Backend_Interface
  */
-require_once 'Zend/Cache/Backend/Interface.php';
+require_once 'Zend/Cache/Backend/ExtendedInterface.php';
 
 /**
  * @see Zend_Cache_Backend
@@ -33,10 +34,10 @@ require_once 'Zend/Cache/Backend.php';
 /**
  * @package    Zend_Cache
  * @subpackage Zend_Cache_Backend
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
-class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache_Backend_Interface
+class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache_Backend_ExtendedInterface
 {
     /**
      * Available options
@@ -83,7 +84,7 @@ class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache
     public function __construct(array $options = array())
     {
         parent::__construct($options);
-        if (is_null($this->_options['cache_db_complete_path'])) {
+        if ($this->_options['cache_db_complete_path'] === null) {
             Zend_Cache::throwException('cache_db_complete_path option has to set');
         }
         if (!extension_loaded('sqlite')) {
@@ -161,7 +162,7 @@ class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache
         $lifetime = $this->getLifetime($specificLifetime);
         $data = @sqlite_escape_string($data);
         $mktime = time();
-        if (is_null($lifetime)) {
+        if ($lifetime === null) {
             $expire = 0;
         } else {
             $expire = $mktime + $lifetime;
@@ -175,7 +176,7 @@ class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache
         }
         $res = true;
         foreach ($tags as $tag) {
-            $res = $res && $this->_registerTag($id, $tag);
+            $res = $this->_registerTag($id, $tag) && $res;
         }
         return $res;
     }
@@ -207,6 +208,8 @@ class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache
      *                                               ($tags can be an array of strings or a single string)
      * Zend_Cache::CLEANING_MODE_NOT_MATCHING_TAG => remove cache entries not {matching one of the given tags}
      *                                               ($tags can be an array of strings or a single string)
+     * Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG => remove cache entries matching any given tags
+     *                                               ($tags can be an array of strings or a single string)
      *
      * @param  string $mode Clean mode
      * @param  array  $tags Array of tags
@@ -220,6 +223,248 @@ class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache
         return $return;
     }
 
+    /**
+     * Return an array of stored cache ids
+     *
+     * @return array array of stored cache ids (string)
+     */
+    public function getIds()
+    {
+        $this->_checkAndBuildStructure();
+        $res = $this->_query("SELECT id FROM cache WHERE (expire=0 OR expire>" . time() . ")");
+        $result = array();
+        while ($id = @sqlite_fetch_single($res)) {
+            $result[] = $id;
+        }
+        return $result;
+    }
+
+    /**
+     * Return an array of stored tags
+     *
+     * @return array array of stored tags (string)
+     */
+    public function getTags()
+    {
+        $this->_checkAndBuildStructure();
+        $res = $this->_query("SELECT DISTINCT(name) AS name FROM tag");
+        $result = array();
+        while ($id = @sqlite_fetch_single($res)) {
+            $result[] = $id;
+        }
+        return $result;
+    }
+
+    /**
+     * Return an array of stored cache ids which match given tags
+     *
+     * In case of multiple tags, a logical AND is made between tags
+     *
+     * @param array $tags array of tags
+     * @return array array of matching cache ids (string)
+     */
+    public function getIdsMatchingTags($tags = array())
+    {
+        $first = true;
+        $ids = array();
+        foreach ($tags as $tag) {
+            $res = $this->_query("SELECT DISTINCT(id) AS id FROM tag WHERE name='$tag'");
+            if (!$res) {
+                return array();
+            }
+            $rows = @sqlite_fetch_all($res, SQLITE_ASSOC);
+            $ids2 = array();
+            foreach ($rows as $row) {
+                $ids2[] = $row['id'];
+            }
+            if ($first) {
+                $ids = $ids2;
+                $first = false;
+            } else {
+                $ids = array_intersect($ids, $ids2);
+            }
+        }
+        $result = array();
+        foreach ($ids as $id) {
+            $result[] = $id;
+        }
+        return $result;
+    }
+
+    /**
+     * Return an array of stored cache ids which don't match given tags
+     *
+     * In case of multiple tags, a logical OR is made between tags
+     *
+     * @param array $tags array of tags
+     * @return array array of not matching cache ids (string)
+     */
+    public function getIdsNotMatchingTags($tags = array())
+    {
+        $res = $this->_query("SELECT id FROM cache");
+        $rows = @sqlite_fetch_all($res, SQLITE_ASSOC);
+        $result = array();
+        foreach ($rows as $row) {
+            $id = $row['id'];
+            $matching = false;
+            foreach ($tags as $tag) {
+                $res = $this->_query("SELECT COUNT(*) AS nbr FROM tag WHERE name='$tag' AND id='$id'");
+                if (!$res) {
+                    return array();
+                }
+                $nbr = (int) @sqlite_fetch_single($res);
+                if ($nbr > 0) {
+                    $matching = true;
+                }
+            }
+            if (!$matching) {
+                $result[] = $id;
+            }
+        }
+        return $result;
+    }
+
+    /**
+     * Return an array of stored cache ids which match any given tags
+     *
+     * In case of multiple tags, a logical AND is made between tags
+     *
+     * @param array $tags array of tags
+     * @return array array of any matching cache ids (string)
+     */
+    public function getIdsMatchingAnyTags($tags = array())
+    {
+        $first = true;
+        $ids = array();
+        foreach ($tags as $tag) {
+            $res = $this->_query("SELECT DISTINCT(id) AS id FROM tag WHERE name='$tag'");
+            if (!$res) {
+                return array();
+            }
+            $rows = @sqlite_fetch_all($res, SQLITE_ASSOC);
+            $ids2 = array();
+            foreach ($rows as $row) {
+                $ids2[] = $row['id'];
+            }
+            if ($first) {
+                $ids = $ids2;
+                $first = false;
+            } else {
+                $ids = array_merge($ids, $ids2);
+            }
+        }
+        $result = array();
+        foreach ($ids as $id) {
+            $result[] = $id;
+        }
+        return $result;
+    }
+
+    /**
+     * Return the filling percentage of the backend storage
+     *
+     * @throws Zend_Cache_Exception
+     * @return int integer between 0 and 100
+     */
+    public function getFillingPercentage()
+    {
+        $dir = dirname($this->_options['cache_db_complete_path']);
+        $free = disk_free_space($dir);
+        $total = disk_total_space($dir);
+        if ($total == 0) {
+            Zend_Cache::throwException('can\'t get disk_total_space');
+        } else {
+            if ($free >= $total) {
+                return 100;
+            }
+            return ((int) (100. * ($total - $free) / $total));
+        }
+    }
+
+    /**
+     * Return an array of metadatas for the given cache id
+     *
+     * The array must include these keys :
+     * - expire : the expire timestamp
+     * - tags : a string array of tags
+     * - mtime : timestamp of last modification time
+     *
+     * @param string $id cache id
+     * @return array array of metadatas (false if the cache id is not found)
+     */
+    public function getMetadatas($id)
+    {
+        $tags = array();
+        $res = $this->_query("SELECT name FROM tag WHERE id='$id'");
+        if ($res) {
+            $rows = @sqlite_fetch_all($res, SQLITE_ASSOC);
+            foreach ($rows as $row) {
+                $tags[] = $row['name'];
+            }
+        }
+        $this->_query('CREATE TABLE cache (id TEXT PRIMARY KEY, content BLOB, lastModified INTEGER, expire INTEGER)');
+        $res = $this->_query("SELECT lastModified,expire FROM cache WHERE id='$id'");
+        if (!$res) {
+            return false;
+        }
+        $row = @sqlite_fetch_array($res, SQLITE_ASSOC);
+        return array(
+            'tags' => $tags,
+            'mtime' => $row['lastModified'],
+            'expire' => $row['expire']
+        );
+    }
+
+    /**
+     * Give (if possible) an extra lifetime to the given cache id
+     *
+     * @param string $id cache id
+     * @param int $extraLifetime
+     * @return boolean true if ok
+     */
+    public function touch($id, $extraLifetime)
+    {
+        $sql = "SELECT expire FROM cache WHERE id='$id' AND (expire=0 OR expire>" . time() . ')';
+        $res = $this->_query($sql);
+        if (!$res) {
+            return false;
+        }
+        $expire = @sqlite_fetch_single($res);
+        $newExpire = $expire + $extraLifetime;
+        $res = $this->_query("UPDATE cache SET lastModified=" . time() . ", expire=$newExpire WHERE id='$id'");
+        if ($res) {
+            return true;
+        } else {
+            return false;
+        }
+    }
+
+    /**
+     * Return an associative array of capabilities (booleans) of the backend
+     *
+     * The array must include these keys :
+     * - automatic_cleaning (is automating cleaning necessary)
+     * - tags (are tags supported)
+     * - expired_read (is it possible to read expired cache records
+     *                 (for doNotTestCacheValidity option for example))
+     * - priority does the backend deal with priority when saving
+     * - infinite_lifetime (is infinite lifetime can work with this backend)
+     * - get_list (is it possible to get the list of cache ids and the complete list of tags)
+     *
+     * @return array associative of with capabilities
+     */
+    public function getCapabilities()
+    {
+        return array(
+            'automatic_cleaning' => true,
+            'tags' => true,
+            'expired_read' => true,
+            'priority' => false,
+            'infinite_lifetime' => true,
+            'get_list' => true
+        );
+    }
+
     /**
      * PUBLIC METHOD FOR UNIT TESTING ONLY !
      *
@@ -285,7 +530,6 @@ class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache
             $rand = rand(1, $this->_options['automatic_vacuum_factor']);
             if ($rand == 1) {
                 $this->_query('VACUUM');
-                @sqlite_close($this->_getConnection());
             }
         }
     }
@@ -360,6 +604,8 @@ class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache
      *                                               ($tags can be an array of strings or a single string)
      * Zend_Cache::CLEANING_MODE_NOT_MATCHING_TAG => remove cache entries not {matching one of the given tags}
      *                                               ($tags can be an array of strings or a single string)
+     * Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG => remove cache entries matching any given tags
+     *                                               ($tags can be an array of strings or a single string)
      *
      * @param  string $mode Clean mode
      * @param  array  $tags Array of tags
@@ -367,65 +613,44 @@ class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache
      */
     private function _clean($mode = Zend_Cache::CLEANING_MODE_ALL, $tags = array())
     {
-        if ($mode==Zend_Cache::CLEANING_MODE_ALL) {
-            $res1 = $this->_query('DELETE FROM cache');
-            $res2 = $this->_query('DELETE FROM tag');
-            return $res1 && $res2;
-        }
-        if ($mode==Zend_Cache::CLEANING_MODE_OLD) {
-            $mktime = time();
-            $res1 = $this->_query("DELETE FROM tag WHERE id IN (SELECT id FROM cache WHERE expire>0 AND expire<=$mktime)");
-            $res2 = $this->_query("DELETE FROM cache WHERE expire>0 AND expire<=$mktime");
-            return $res1 && $res2;
-        }
-        if ($mode==Zend_Cache::CLEANING_MODE_MATCHING_TAG) {
-            $first = true;
-            $ids = array();
-            foreach ($tags as $tag) {
-                $res = $this->_query("SELECT DISTINCT(id) AS id FROM tag WHERE name='$tag'");
-                if (!$res) {
-                    return false;
+        switch ($mode) {
+            case Zend_Cache::CLEANING_MODE_ALL:
+                $res1 = $this->_query('DELETE FROM cache');
+                $res2 = $this->_query('DELETE FROM tag');
+                return $res1 && $res2;
+                break;
+            case Zend_Cache::CLEANING_MODE_OLD:
+                $mktime = time();
+                $res1 = $this->_query("DELETE FROM tag WHERE id IN (SELECT id FROM cache WHERE expire>0 AND expire<=$mktime)");
+                $res2 = $this->_query("DELETE FROM cache WHERE expire>0 AND expire<=$mktime");
+                return $res1 && $res2;
+                break;
+            case Zend_Cache::CLEANING_MODE_MATCHING_TAG:
+                $ids = $this->getIdsMatchingTags($tags);
+                $result = true;
+                foreach ($ids as $id) {
+                    $result = $this->remove($id) && $result;
                 }
-                $rows = @sqlite_fetch_all($res, SQLITE_ASSOC);
-                $ids2 = array();
-                foreach ($rows as $row) {
-                    $ids2[] = $row['id'];
+                return $result;
+                break;
+            case Zend_Cache::CLEANING_MODE_NOT_MATCHING_TAG:
+                $ids = $this->getIdsNotMatchingTags($tags);
+                $result = true;
+                foreach ($ids as $id) {
+                    $result = $this->remove($id) && $result;
                 }
-                if ($first) {
-                    $ids = $ids2;
-                    $first = false;
-                } else {
-                    $ids = array_intersect($ids, $ids2);
+                return $result;
+                break;
+            case Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG:
+                $ids = $this->getIdsMatchingAnyTags($tags);
+                $result = true;
+                foreach ($ids as $id) {
+                    $result = $this->remove($id) && $result;
                 }
-            }
-            $result = true;
-            foreach ($ids as $id) {
-                $result = $result && ($this->remove($id));
-            }
-            return $result;
-        }
-        if ($mode==Zend_Cache::CLEANING_MODE_NOT_MATCHING_TAG) {
-            $res = $this->_query("SELECT id FROM cache");
-            $rows = @sqlite_fetch_all($res, SQLITE_ASSOC);
-            $result = true;
-            foreach ($rows as $row) {
-                $id = $row['id'];
-                $matching = false;
-                foreach ($tags as $tag) {
-                    $res = $this->_query("SELECT COUNT(*) AS nbr FROM tag WHERE name='$tag' AND id='$id'");
-                    if (!$res) {
-                        return false;
-                    }
-                    $nbr = (int) @sqlite_fetch_single($res);
-                    if ($nbr > 0) {
-                        $matching = true;
-                    }
-                }
-                if (!$matching) {
-                    $result = $result && $this->remove($id);
-                }
-            }
-            return $result;
+                return $result;
+                break;
+            default:
+                break;
         }
         return false;
     }
@@ -433,6 +658,7 @@ class Zend_Cache_Backend_Sqlite extends Zend_Cache_Backend implements Zend_Cache
     /**
      * Check if the database structure is ok (with the good version), if no : build it
      *
+     * @throws Zend_Cache_Exception
      * @return boolean True if ok
      */
     private function _checkAndBuildStructure()
