@@ -14,8 +14,9 @@
  *
  * @category   Zend
  * @package    Zend_Controller
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Front.php 24593 2012-01-05 20:35:02Z matthew $
  */
 
 
@@ -25,16 +26,13 @@ require_once 'Zend/Loader.php';
 /** Zend_Controller_Action_HelperBroker */
 require_once 'Zend/Controller/Action/HelperBroker.php';
 
-/** Zend_Controller_Exception */
-require_once 'Zend/Controller/Exception.php';
-
 /** Zend_Controller_Plugin_Broker */
 require_once 'Zend/Controller/Plugin/Broker.php';
 
 /**
  * @category   Zend
  * @package    Zend_Controller
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Controller_Front
@@ -135,8 +133,8 @@ class Zend_Controller_Front
     }
 
     /**
-     * Enforce singleton; disallow cloning 
-     * 
+     * Enforce singleton; disallow cloning
+     *
      * @return void
      */
     private function __clone()
@@ -265,9 +263,9 @@ class Zend_Controller_Front
     }
 
     /**
-     * Remove a controller directory by module name 
-     * 
-     * @param  string $module 
+     * Remove a controller directory by module name
+     *
+     * @param  string $module
      * @return bool
      */
     public function removeControllerDirectory($module)
@@ -289,8 +287,9 @@ class Zend_Controller_Front
     {
         try{
             $dir = new DirectoryIterator($path);
-        }catch(Exception $e){
-            throw new Zend_Controller_Exception("Directory $path not readable");
+        } catch(Exception $e) {
+            require_once 'Zend/Controller/Exception.php';
+            throw new Zend_Controller_Exception("Directory $path not readable", 0, $e);
         }
         foreach ($dir as $file) {
             if ($file->isDot() || !$file->isDir()) {
@@ -313,8 +312,8 @@ class Zend_Controller_Front
 
     /**
      * Return the path to a module directory (but not the controllers directory within)
-     * 
-     * @param  string $module 
+     *
+     * @param  string $module
      * @return string|null
      */
     public function getModuleDirectory($module = null)
@@ -444,10 +443,14 @@ class Zend_Controller_Front
     public function setRequest($request)
     {
         if (is_string($request)) {
-            Zend_Loader::loadClass($request);
+            if (!class_exists($request)) {
+                require_once 'Zend/Loader.php';
+                Zend_Loader::loadClass($request);
+            }
             $request = new $request();
         }
         if (!$request instanceof Zend_Controller_Request_Abstract) {
+            require_once 'Zend/Controller/Exception.php';
             throw new Zend_Controller_Exception('Invalid request class');
         }
 
@@ -482,11 +485,15 @@ class Zend_Controller_Front
     public function setRouter($router)
     {
         if (is_string($router)) {
-            Zend_Loader::loadClass($router);
+            if (!class_exists($router)) {
+                require_once 'Zend/Loader.php';
+                Zend_Loader::loadClass($router);
+            }
             $router = new $router();
         }
 
         if (!$router instanceof Zend_Controller_Router_Interface) {
+            require_once 'Zend/Controller/Exception.php';
             throw new Zend_Controller_Exception('Invalid router class');
         }
 
@@ -536,6 +543,7 @@ class Zend_Controller_Front
     public function setBaseUrl($base = null)
     {
         if (!is_string($base) && (null !== $base)) {
+            require_once 'Zend/Controller/Exception.php';
             throw new Zend_Controller_Exception('Rewrite base must be a string');
         }
 
@@ -609,10 +617,14 @@ class Zend_Controller_Front
     public function setResponse($response)
     {
         if (is_string($response)) {
-            Zend_Loader::loadClass($response);
+            if (!class_exists($response)) {
+                require_once 'Zend/Loader.php';
+                Zend_Loader::loadClass($response);
+            }
             $response = new $response();
         }
         if (!$response instanceof Zend_Controller_Response_Abstract) {
+            require_once 'Zend/Controller/Exception.php';
             throw new Zend_Controller_Exception('Invalid response class');
         }
 
@@ -775,8 +787,8 @@ class Zend_Controller_Front
      * Default behaviour is to trap them in the response object; call this
      * method to have them thrown.
      *
-     * Passing no value will return the current value of the flag; passing a 
-     * boolean true or false value will set the flag and return the current 
+     * Passing no value will return the current value of the flag; passing a
+     * boolean true or false value will set the flag and return the current
      * object instance.
      *
      * @param boolean $flag Defaults to null (return flag state)
@@ -895,7 +907,15 @@ class Zend_Controller_Front
             */
             $this->_plugins->routeStartup($this->_request);
 
-            $router->route($this->_request);
+            try {
+                $router->route($this->_request);
+            }  catch (Exception $e) {
+                if ($this->throwExceptions()) {
+                    throw $e;
+                }
+
+                $this->_response->setException($e);
+            }
 
             /**
             * Notify plugins of router completion
