@@ -14,8 +14,9 @@
  *
  * @category   Zend
  * @package    Zend_Controller
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Abstract.php 24593 2012-01-05 20:35:02Z matthew $
  */
 
 /**
@@ -25,7 +26,7 @@
  *
  * @package Zend_Controller
  * @subpackage Response
- * @copyright  Copyright (c) 2005-2007 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 abstract class Zend_Controller_Response_Abstract
@@ -82,6 +83,22 @@ abstract class Zend_Controller_Response_Abstract
      */
     public $headersSentThrowsException = true;
 
+    /**
+     * Normalize a header name
+     *
+     * Normalizes a header name to X-Capitalized-Names
+     *
+     * @param  string $name
+     * @return string
+     */
+    protected function _normalizeHeader($name)
+    {
+        $filtered = str_replace(array('-', '_'), ' ', (string) $name);
+        $filtered = ucwords(strtolower($filtered));
+        $filtered = str_replace(' ', '-', $filtered);
+        return $filtered;
+    }
+
     /**
      * Set a header
      *
@@ -96,7 +113,7 @@ abstract class Zend_Controller_Response_Abstract
     public function setHeader($name, $value, $replace = false)
     {
         $this->canSendHeaders(true);
-        $name  = (string) $name;
+        $name  = $this->_normalizeHeader($name);
         $value = (string) $value;
 
         if ($replace) {
@@ -167,6 +184,27 @@ abstract class Zend_Controller_Response_Abstract
         return $this;
     }
 
+    /**
+     * Clears the specified HTTP header
+     *
+     * @param  string $name
+     * @return Zend_Controller_Response_Abstract
+     */
+    public function clearHeader($name)
+    {
+        if (! count($this->_headers)) {
+            return $this;
+        }
+
+        foreach ($this->_headers as $index => $header) {
+            if ($name == $header['name']) {
+                unset($this->_headers[$index]);
+            }
+        }
+
+        return $this;
+    }
+
     /**
      * Set raw HTTP header
      *
@@ -206,6 +244,26 @@ abstract class Zend_Controller_Response_Abstract
         return $this;
     }
 
+    /**
+     * Clears the specified raw HTTP header
+     *
+     * @param  string $headerRaw
+     * @return Zend_Controller_Response_Abstract
+     */
+    public function clearRawHeader($headerRaw)
+    {
+        if (! count($this->_headersRaw)) {
+            return $this;
+        }
+
+        $key = array_search($headerRaw, $this->_headersRaw);
+        if ($key !== false) {
+            unset($this->_headersRaw[$key]);
+        }
+
+        return $this;
+    }
+
     /**
      * Clear all headers, normal and raw
      *
@@ -526,9 +584,8 @@ abstract class Zend_Controller_Response_Abstract
      */
     public function outputBody()
     {
-        foreach ($this->_body as $content) {
-            echo $content;
-        }
+        $body = implode('', $this->_body);
+        echo $body;
     }
 
     /**
