@@ -14,7 +14,7 @@
  *
  * @category   Zend
  * @package    Zend_Form
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
@@ -22,19 +22,19 @@
 require_once 'Zend/Form/Element/Xhtml.php';
 
 /**
- * Zend_Form_Element 
- * 
+ * Zend_Form_Element
+ *
  * @category   Zend
  * @package    Zend_Form
  * @subpackage Element
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: File.php 11705 2008-10-06 19:51:46Z thomas $
+ * @version    $Id: File.php 24848 2012-05-31 19:28:48Z rob $
  */
 class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 {
     /**
-     * @const string Plugin loader type
+     * Plugin loader type
      */
     const TRANSFER_ADAPTER = 'TRANSFER_ADAPTER';
 
@@ -53,6 +53,11 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
      */
     protected $_validated = false;
 
+    /**
+     * @var boolean Disable value to be equal to file content
+     */
+    protected $_valueDisabled = false;
+
     /**
      * @var integer Internal multifile counter
      */
@@ -61,33 +66,35 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
     /**
      * @var integer Maximum file size for MAX_FILE_SIZE attribut of form
      */
-    protected static $_maxFileSize = 0;
+    protected static $_maxFileSize = -1;
 
     /**
      * Load default decorators
-     * 
-     * @return void
+     *
+     * @return Zend_Form_Element_File
      */
     public function loadDefaultDecorators()
     {
         if ($this->loadDefaultDecoratorsIsDisabled()) {
-            return;
+            return $this;
         }
 
         $decorators = $this->getDecorators();
         if (empty($decorators)) {
             $this->addDecorator('File')
                  ->addDecorator('Errors')
+                 ->addDecorator('Description', array('tag' => 'p', 'class' => 'description'))
                  ->addDecorator('HtmlTag', array('tag' => 'dd'))
                  ->addDecorator('Label', array('tag' => 'dt'));
         }
+        return $this;
     }
 
     /**
      * Set plugin loader
-     * 
-     * @param  Zend_Loader_PluginLoader_Interface $loader 
-     * @param  string $type 
+     *
+     * @param  Zend_Loader_PluginLoader_Interface $loader
+     * @param  string $type
      * @return Zend_Form_Element_File
      */
     public function setPluginLoader(Zend_Loader_PluginLoader_Interface $loader, $type)
@@ -104,8 +111,8 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Get Plugin Loader
-     * 
-     * @param  string $type 
+     *
+     * @param  string $type
      * @return Zend_Loader_PluginLoader_Interface
      */
     public function getPluginLoader($type)
@@ -129,10 +136,10 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Add prefix path for plugin loader
-     * 
-     * @param  string $prefix 
-     * @param  string $path 
-     * @param  string $type 
+     *
+     * @param  string $prefix
+     * @param  string $path
+     * @param  string $type
      * @return Zend_Form_Element_File
      */
     public function addPrefixPath($prefix, $path, $type = null)
@@ -143,9 +150,10 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
         }
 
         if (empty($type)) {
-            $pluginPrefix = rtrim($prefix, '_') . '_Transfer_Adapter';
+            $nsSeparator = (false !== strpos($prefix, '\\'))?'\\':'_';
+            $pluginPrefix = rtrim($prefix, $nsSeparator) . $nsSeparator . 'Transfer' . $nsSeparator . 'Adapter';
             $pluginPath   = rtrim($path, DIRECTORY_SEPARATOR) . '/Transfer/Adapter/';
-            $loader    = $this->getPluginLoader(self::TRANSFER_ADAPTER);
+            $loader       = $this->getPluginLoader(self::TRANSFER_ADAPTER);
             $loader->addPrefixPath($pluginPrefix, $pluginPath);
             return parent::addPrefixPath($prefix, $path, null);
         }
@@ -157,8 +165,8 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Set transfer adapter
-     * 
-     * @param  string|Zend_File_Transfer_Adapter_Abstract $adapter 
+     *
+     * @param  string|Zend_File_Transfer_Adapter_Abstract $adapter
      * @return Zend_Form_Element_File
      */
     public function setTransferAdapter($adapter)
@@ -174,6 +182,11 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
             throw new Zend_Form_Element_Exception('Invalid adapter specified');
         }
 
+        foreach (array('filter', 'validate') as $type) {
+            $loader = $this->getPluginLoader($type);
+            $this->_adapter->setPluginLoader($loader, $type);
+        }
+
         return $this;
     }
 
@@ -181,7 +194,7 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
      * Get transfer adapter
      *
      * Lazy loads HTTP transfer adapter when no adapter registered.
-     * 
+     *
      * @return Zend_File_Transfer_Adapter_Abstract
      */
     public function getTransferAdapter()
@@ -194,10 +207,10 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Add Validator; proxy to adapter
-     * 
-     * @param  string|Zend_Validate_Interface $validator 
-     * @param  bool $breakChainOnFailure 
-     * @param  mixed $options 
+     *
+     * @param  string|Zend_Validate_Interface $validator
+     * @param  bool $breakChainOnFailure
+     * @param  mixed $options
      * @return Zend_Form_Element_File
      */
     public function addValidator($validator, $breakChainOnFailure = false, $options = array())
@@ -211,8 +224,8 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Add multiple validators at once; proxy to adapter
-     * 
-     * @param  array $validators 
+     *
+     * @param  array $validators
      * @return Zend_Form_Element_File
      */
     public function addValidators(array $validators)
@@ -226,8 +239,8 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Add multiple validators at once, overwriting; proxy to adapter
-     * 
-     * @param  array $validators 
+     *
+     * @param  array $validators
      * @return Zend_Form_Element_File
      */
     public function setValidators(array $validators)
@@ -241,31 +254,36 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Retrieve validator by name; proxy to adapter
-     * 
-     * @param  string $name 
+     *
+     * @param  string $name
      * @return Zend_Validate_Interface|null
      */
     public function getValidator($name)
     {
-        $adapter = $this->getTransferAdapter();
+        $adapter    = $this->getTransferAdapter();
         return $adapter->getValidator($name);
     }
 
     /**
      * Retrieve all validators; proxy to adapter
-     * 
+     *
      * @return array
      */
     public function getValidators()
     {
         $adapter = $this->getTransferAdapter();
-        return $adapter->getValidators($this->getName());
+        $validators = $adapter->getValidators($this->getName());
+        if ($validators === null) {
+            $validators = array();
+        }
+
+        return $validators;
     }
 
     /**
      * Remove validator by name; proxy to adapter
-     * 
-     * @param  string $name 
+     *
+     * @param  string $name
      * @return Zend_Form_Element_File
      */
     public function removeValidator($name)
@@ -279,7 +297,7 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Remove all validators; proxy to adapter
-     * 
+     *
      * @return Zend_Form_Element_File
      */
     public function clearValidators()
@@ -293,7 +311,7 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Add Filter; proxy to adapter
-     * 
+     *
      * @param  string|array $filter  Type of filter to add
      * @param  string|array $options Options to set for the filter
      * @return Zend_Form_Element_File
@@ -308,8 +326,8 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Add Multiple filters at once; proxy to adapter
-     * 
-     * @param  array $filters 
+     *
+     * @param  array $filters
      * @return Zend_Form_Element_File
      */
     public function addFilters(array $filters)
@@ -336,8 +354,8 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Retrieve individual filter; proxy to adapter
-     * 
-     * @param  string $name 
+     *
+     * @param  string $name
      * @return Zend_Filter_Interface|null
      */
     public function getFilter($name)
@@ -349,18 +367,23 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
     /**
      * Returns all set filters; proxy to adapter
      *
-     * @return null|array List of set filters
+     * @return array List of set filters
      */
     public function getFilters()
     {
         $adapter = $this->getTransferAdapter();
-        return $adapter->getFilters($this->getName());
+        $filters = $adapter->getFilters($this->getName());
+
+        if ($filters === null) {
+            $filters = array();
+        }
+        return $filters;
     }
 
     /**
-     * Remove an individual filter; proxy to adapter 
-     * 
-     * @param  string $name 
+     * Remove an individual filter; proxy to adapter
+     *
+     * @param  string $name
      * @return Zend_Form_Element_File
      */
     public function removeFilter($name)
@@ -373,7 +396,7 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Remove all filters; proxy to adapter
-     * 
+     *
      * @return Zend_Form_Element_File
      */
     public function clearFilters()
@@ -386,9 +409,9 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Validate upload
-     * 
+     *
      * @param  string $value   File, can be optional, give null to validate all files
-     * @param  mixed  $context 
+     * @param  mixed  $context
      * @return bool
      */
     public function isValid($value, $context = null)
@@ -397,19 +420,18 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
             return true;
         }
 
-        $adapter = $this->getTransferAdapter();
+        $adapter    = $this->getTransferAdapter();
+        $translator = $this->getTranslator();
+        if ($translator !== null) {
+            $adapter->setTranslator($translator);
+        }
 
         if (!$this->isRequired()) {
-            $adapter->setOptions(array('ignoreNoFile' => true));
+            $adapter->setOptions(array('ignoreNoFile' => true), $this->getName());
         } else {
-            $adapter->setOptions(array('ignoreNoFile' => false));
-            if ($this->autoInsertNotEmptyValidator() and
-                   !$this->getValidator('NotEmpty'))
-            {
-                $validators = $this->getValidators();
-                $notEmpty   = array('validator' => 'NotEmpty', 'breakChainOnFailure' => true);
-                array_unshift($validators, $notEmpty);
-                $this->setValidators($validators);
+            $adapter->setOptions(array('ignoreNoFile' => false), $this->getName());
+            if ($this->autoInsertNotEmptyValidator() && !$this->getValidator('NotEmpty')) {
+                $this->addValidator = array('validator' => 'NotEmpty', 'breakChainOnFailure' => true);
             }
         }
 
@@ -425,13 +447,12 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
     /**
      * Receive the uploaded file
      *
-     * @param  string $value
      * @return boolean
      */
-    public function receive($value = null)
+    public function receive()
     {
         if (!$this->_validated) {
-            if (!$this->isValid($value)) {
+            if (!$this->isValid($this->getName())) {
                 return false;
             }
         }
@@ -446,38 +467,38 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Retrieve error codes; proxy to transfer adapter
-     * 
+     *
      * @return array
      */
     public function getErrors()
     {
-        return $this->getTransferAdapter()->getErrors();
+        return parent::getErrors() + $this->getTransferAdapter()->getErrors();
     }
 
     /**
      * Are there errors registered?
-     * 
+     *
      * @return bool
      */
     public function hasErrors()
     {
-        return $this->getTransferAdapter()->hasErrors();
+        return (parent::hasErrors() || $this->getTransferAdapter()->hasErrors());
     }
 
     /**
      * Retrieve error messages; proxy to transfer adapter
-     * 
+     *
      * @return array
      */
     public function getMessages()
     {
-        return $this->getTransferAdapter()->getMessages();
+        return parent::getMessages() + $this->getTransferAdapter()->getMessages();
     }
 
     /**
      * Set the upload destination
-     * 
-     * @param  string $path 
+     *
+     * @param  string $path
      * @return Zend_Form_Element_File
      */
     public function setDestination($path)
@@ -488,7 +509,7 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Get the upload destination
-     * 
+     *
      * @return string
      */
     public function getDestination()
@@ -498,23 +519,40 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
 
     /**
      * Get the final filename
-     * 
-     * @param  string $value (Optional) Element or file to return
+     *
+     * @param  string  $value (Optional) Element or file to return
+     * @param  boolean $path  (Optional) Return also the path, defaults to true
      * @return string
      */
-    public function getFileName($value = null)
+    public function getFileName($value = null, $path = true)
     {
         if (empty($value)) {
             $value = $this->getName();
         }
-        return $this->getTransferAdapter()->getFileName($value);
+
+        return $this->getTransferAdapter()->getFileName($value, $path);
+    }
+
+    /**
+     * Get internal file informations
+     *
+     * @param  string $value (Optional) Element or file to return
+     * @return array
+     */
+    public function getFileInfo($value = null)
+    {
+        if (empty($value)) {
+            $value = $this->getName();
+        }
+
+        return $this->getTransferAdapter()->getFileInfo($value);
     }
 
     /**
      * Set a multifile element
      *
      * @param integer $count Number of file elements
-     * @return Zend_Form_Element_File Provides fluid interface
+     * @return Zend_Form_Element_File Provides fluent interface
      */
     public function setMultiFile($count)
     {
@@ -542,22 +580,329 @@ class Zend_Form_Element_File extends Zend_Form_Element_Xhtml
     /**
      * Sets the maximum file size of the form
      *
-     * @param  integer $size
      * @return integer
      */
-    public function setMaxFileSize($size)
+    public function getMaxFileSize()
     {
-        self::$_maxFileSize = $size;
-        return $this;
+        if (self::$_maxFileSize < 0) {
+            $ini = $this->_convertIniToInteger(trim(ini_get('post_max_size')));
+            $max = $this->_convertIniToInteger(trim(ini_get('upload_max_filesize')));
+            $min = max($ini, $max);
+            if ($ini > 0) {
+                $min = min($min, $ini);
+            }
+
+            if ($max > 0) {
+                $min = min($min, $max);
+            }
+
+            self::$_maxFileSize = $min;
+        }
+
+        return self::$_maxFileSize;
     }
 
     /**
      * Sets the maximum file size of the form
      *
+     * @param  integer $size
      * @return integer
      */
-    public function getMaxFileSize()
+    public function setMaxFileSize($size)
     {
-        return self::$_maxFileSize;
+        $ini = $this->_convertIniToInteger(trim(ini_get('post_max_size')));
+        $max = $this->_convertIniToInteger(trim(ini_get('upload_max_filesize')));
+
+        if (($max > -1) && ($size > $max)) {
+            trigger_error("Your 'upload_max_filesize' config setting limits the maximum filesize to '$max'. You tried to set '$size'.", E_USER_NOTICE);
+            $size = $max;
+        }
+
+        if (($ini > -1) && ($size > $ini)) {
+            trigger_error("Your 'post_max_size' config setting limits the maximum filesize to '$ini'. You tried to set '$size'.", E_USER_NOTICE);
+            $size = $ini;
+        }
+
+        self::$_maxFileSize = $size;
+        return $this;
+    }
+
+    /**
+     * Converts a ini setting to a integer value
+     *
+     * @param  string $setting
+     * @return integer
+     */
+    private function _convertIniToInteger($setting)
+    {
+        if (!is_numeric($setting)) {
+            $type = strtoupper(substr($setting, -1));
+            $setting = (integer) substr($setting, 0, -1);
+
+            switch ($type) {
+                case 'K' :
+                    $setting *= 1024;
+                    break;
+
+                case 'M' :
+                    $setting *= 1024 * 1024;
+                    break;
+
+                case 'G' :
+                    $setting *= 1024 * 1024 * 1024;
+                    break;
+
+                default :
+                    break;
+            }
+        }
+
+        return (integer) $setting;
+    }
+
+    /**
+     * Set if the file will be uploaded when getting the value
+     * This defaults to false which will force receive() when calling getValues()
+     *
+     * @param boolean $flag Sets if the file is handled as the elements value
+     * @return Zend_Form_Element_File
+     */
+    public function setValueDisabled($flag)
+    {
+        $this->_valueDisabled = (bool) $flag;
+        return $this;
+    }
+
+    /**
+     * Returns if the file will be uploaded when calling getValues()
+     *
+     * @return boolean Receive the file on calling getValues()?
+     */
+    public function isValueDisabled()
+    {
+        return $this->_valueDisabled;
+    }
+
+    /**
+     * Processes the file, returns null or the filename only
+     * For the complete path, use getFileName
+     *
+     * @return null|string
+     */
+    public function getValue()
+    {
+        if ($this->_value !== null) {
+            return $this->_value;
+        }
+
+        $content = $this->getTransferAdapter()->getFileName($this->getName());
+        if (empty($content)) {
+            return null;
+        }
+
+        if (!$this->isValid(null)) {
+            return null;
+        }
+
+        if (!$this->_valueDisabled && !$this->receive()) {
+            return null;
+        }
+
+        return $this->getFileName(null, false);
+    }
+
+    /**
+     * Disallow setting the value
+     *
+     * @param  mixed $value
+     * @return Zend_Form_Element_File
+     */
+    public function setValue($value)
+    {
+        return $this;
+    }
+
+    /**
+     * Set translator object for localization
+     *
+     * @param  Zend_Translate|null $translator
+     * @return Zend_Form_Element_File
+     */
+    public function setTranslator($translator = null)
+    {
+        $adapter = $this->getTransferAdapter();
+        $adapter->setTranslator($translator);
+        parent::setTranslator($translator);
+
+        return $this;
+    }
+
+    /**
+     * Retrieve localization translator object
+     *
+     * @return Zend_Translate_Adapter|null
+     */
+    public function getTranslator()
+    {
+        if ($this->translatorIsDisabled()) {
+            return null;
+        }
+
+        $translator = $this->getTransferAdapter()->getTranslator();
+        if (null === $translator) {
+            require_once 'Zend/Form.php';
+            return Zend_Form::getDefaultTranslator();
+        }
+
+        return $translator;
+    }
+
+    /**
+     * Indicate whether or not translation should be disabled
+     *
+     * @param  bool $flag
+     * @return Zend_Form_Element_File
+     */
+    public function setDisableTranslator($flag)
+    {
+        $adapter = $this->getTransferAdapter();
+        $adapter->setDisableTranslator($flag);
+        $this->_translatorDisabled = (bool) $flag;
+
+        return $this;
+    }
+
+    /**
+     * Is translation disabled?
+     *
+     * @return bool
+     */
+    public function translatorIsDisabled()
+    {
+        $adapter = $this->getTransferAdapter();
+        return $adapter->translatorIsDisabled();
+    }
+
+    /**
+     * Was the file received?
+     *
+     * @return bool
+     */
+    public function isReceived()
+    {
+        $adapter = $this->getTransferAdapter();
+        return $adapter->isReceived($this->getName());
+    }
+
+    /**
+     * Was the file uploaded?
+     *
+     * @return bool
+     */
+    public function isUploaded()
+    {
+        $adapter = $this->getTransferAdapter();
+        return $adapter->isUploaded($this->getName());
+    }
+
+    /**
+     * Has the file been filtered?
+     *
+     * @return bool
+     */
+    public function isFiltered()
+    {
+        $adapter = $this->getTransferAdapter();
+        return $adapter->isFiltered($this->getName());
+    }
+
+    /**
+     * Returns the hash for this file element
+     *
+     * @param string $hash (Optional) Hash algorithm to use
+     * @return string|array Hashstring
+     */
+    public function getHash($hash = 'crc32')
+    {
+        $adapter = $this->getTransferAdapter();
+        return $adapter->getHash($hash, $this->getName());
+    }
+
+    /**
+     * Returns the filesize for this file element
+     *
+     * @return string|array Filesize
+     */
+    public function getFileSize()
+    {
+        $adapter = $this->getTransferAdapter();
+        return $adapter->getFileSize($this->getName());
+    }
+
+    /**
+     * Returns the mimetype for this file element
+     *
+     * @return string|array Mimetype
+     */
+    public function getMimeType()
+    {
+        $adapter = $this->getTransferAdapter();
+        return $adapter->getMimeType($this->getName());
+    }
+
+    /**
+     * Render form element
+     * Checks for decorator interface to prevent errors
+     *
+     * @param  Zend_View_Interface $view
+     * @return string
+     */
+    public function render(Zend_View_Interface $view = null)
+    {
+        $marker = false;
+        foreach ($this->getDecorators() as $decorator) {
+            if ($decorator instanceof Zend_Form_Decorator_Marker_File_Interface) {
+                $marker = true;
+            }
+        }
+
+        if (!$marker) {
+            require_once 'Zend/Form/Element/Exception.php';
+            throw new Zend_Form_Element_Exception('No file decorator found... unable to render file element');
+        }
+
+        return parent::render($view);
+    }
+
+    /**
+     * Retrieve error messages and perform translation and value substitution
+     *
+     * @return array
+     */
+    protected function _getErrorMessages()
+    {
+        $translator = $this->getTranslator();
+        $messages   = $this->getErrorMessages();
+        $value      = $this->getFileName();
+        foreach ($messages as $key => $message) {
+            if (null !== $translator) {
+                $message = $translator->translate($message);
+            }
+
+            if ($this->isArray() || is_array($value)) {
+                $aggregateMessages = array();
+                foreach ($value as $val) {
+                    $aggregateMessages[] = str_replace('%value%', $val, $message);
+                }
+
+                if (!empty($aggregateMessages)) {
+                    $messages[$key] = $aggregateMessages;
+                }
+            } else {
+                $messages[$key] = str_replace('%value%', $value, $message);
+            }
+        }
+
+        return $messages;
     }
 }
