@@ -15,8 +15,10 @@
  *
  * @category   Zend
  * @package    Zend_Gdata
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage Gdata
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
+ * @version    $Id: Gdata.php 24593 2012-01-05 20:35:02Z matthew $
  */
 
 /**
@@ -29,13 +31,14 @@ require_once 'Zend/Gdata/App.php';
  * Subclasses exist to implement service-specific features
  *
  * As the Google data API protocol is based upon the Atom Publishing Protocol
- * (APP), GData functionality extends the appropriate Zend_Gdata_App classes
+ * (APP), Gdata functionality extends the appropriate Zend_Gdata_App classes
  *
  * @link http://code.google.com/apis/gdata/overview.html
  *
  * @category   Zend
  * @package    Zend_Gdata
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @subpackage Gdata
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Gdata extends Zend_Gdata_App
@@ -68,14 +71,16 @@ class Zend_Gdata extends Zend_Gdata_App
             'Zend_Gdata_App');
 
     /**
-     * Namespaces used for GData data
+     * Namespaces used for Gdata data
      *
      * @var array
      */
     public static $namespaces = array(
-        'openSearch' => 'http://a9.com/-/spec/opensearchrss/1.0/',
-        'rss' => 'http://blogs.law.harvard.edu/tech/rss',
-        'gd' => 'http://schemas.google.com/g/2005');
+        array('gd', 'http://schemas.google.com/g/2005', 1, 0),
+        array('openSearch', 'http://a9.com/-/spec/opensearchrss/1.0/', 1, 0),
+        array('openSearch', 'http://a9.com/-/spec/opensearch/1.1/', 2, 0),
+        array('rss', 'http://blogs.law.harvard.edu/tech/rss', 1, 0)
+    );
 
     /**
      * Client object used to communicate
@@ -95,7 +100,8 @@ class Zend_Gdata extends Zend_Gdata_App
      * Create Gdata object
      *
      * @param Zend_Http_Client $client
-     * @param string $applicationId The identity of the app in the form of Company-AppName-Version
+     * @param string $applicationId The identity of the app in the form of
+     *          Company-AppName-Version
      */
     public function __construct($client = null, $applicationId = 'MyCompany-MyApp-1.0')
     {
@@ -109,15 +115,20 @@ class Zend_Gdata extends Zend_Gdata_App
      * @param  Zend_Http_Client $client The client used for communication
      * @param  string $className The class which is used as the return type
      * @throws Zend_Gdata_App_Exception
-     * @return Zend_Gdata_App_Feed
+     * @return string|Zend_Gdata_App_Feed Returns string only if the object
+     *                                    mapping has been disabled explicitly
+     *                                    by passing false to the
+     *                                    useObjectMapping() function.
      */
-    public static function import($uri, $client = null, $className='Zend_Gdata_Feed')
+    public static function import($uri, $client = null,
+        $className='Zend_Gdata_Feed')
     {
         $app = new Zend_Gdata($client);
         $requestData = $app->decodeRequest('GET', $uri);
         $response = $app->performHttpRequest($requestData['method'], $requestData['url']);
 
         $feedContent = $response->getBody();
+
         $feed = self::importString($feedContent, $className);
         if ($client != null) {
             $feed->setHttpClient($client);
@@ -126,12 +137,15 @@ class Zend_Gdata extends Zend_Gdata_App
     }
 
     /**
-     * Retreive feed object
+     * Retrieve feed as string or object
      *
      * @param mixed $location The location as string or Zend_Gdata_Query
      * @param string $className The class type to use for returning the feed
      * @throws Zend_Gdata_App_InvalidArgumentException
-     * @return Zend_Gdata_Feed
+     * @return string|Zend_Gdata_App_Feed Returns string only if the object
+     *                                    mapping has been disabled explicitly
+     *                                    by passing false to the
+     *                                    useObjectMapping() function.
      */
     public function getFeed($location, $className='Zend_Gdata_Feed')
     {
@@ -149,10 +163,14 @@ class Zend_Gdata extends Zend_Gdata_App
     }
 
     /**
-     * Retreive entry object
+     * Retrieve entry as string or object
      *
      * @param mixed $location The location as string or Zend_Gdata_Query
-     * @return Zend_Gdata_Feed
+     * @throws Zend_Gdata_App_InvalidArgumentException
+     * @return string|Zend_Gdata_App_Entry Returns string only if the object
+     *                                     mapping has been disabled explicitly
+     *                                     by passing false to the
+     *                                     useObjectMapping() function.
      */
     public function getEntry($location, $className='Zend_Gdata_Entry')
     {
@@ -171,11 +189,11 @@ class Zend_Gdata extends Zend_Gdata_App
 
     /**
      * Performs a HTTP request using the specified method.
-     * 
+     *
      * Overrides the definition in the parent (Zend_Gdata_App)
      * and uses the Zend_Gdata_HttpClient functionality
      * to filter the HTTP requests and responses.
-     *  
+     *
      * @param string $method The HTTP method for the request -
      *                       'GET', 'POST', 'PUT', 'DELETE'
      * @param string $url The URL to which this request is being performed,
@@ -204,4 +222,20 @@ class Zend_Gdata extends Zend_Gdata_App
         }
     }
 
+    /**
+     * Determines whether service object is authenticated.
+     *
+     * @return boolean True if service object is authenticated, false otherwise.
+     */
+    public function isAuthenticated()
+    {
+        $client = parent::getHttpClient();
+        if ($client->getClientLoginToken() ||
+            $client->getAuthSubToken()) {
+                return true;
+        }
+
+        return false;
+    }
+
 }
