@@ -15,9 +15,9 @@
  * @category   Zend
  * @package    Zend_Search_Lucene
  * @subpackage Document
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Html.php 18488 2009-10-05 17:11:33Z alexander $
+ * @version    $Id: Html.php 24593 2012-01-05 20:35:02Z matthew $
  */
 
 
@@ -31,7 +31,7 @@ require_once 'Zend/Search/Lucene/Document.php';
  * @category   Zend
  * @package    Zend_Search_Lucene
  * @subpackage Document
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
@@ -67,6 +67,17 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
      */
     private static $_excludeNoFollowLinks = false;
 
+    /**
+     *
+     * List of inline tags
+     *
+     * @var array
+     */
+    private $_inlineTags = array('a', 'abbr', 'acronym', 'dfn', 'em', 'strong', 'code',
+                                'samp', 'kbd', 'var', 'b', 'i', 'big', 'small', 'strike',
+                                'tt', 'u', 'font', 'span', 'bdo', 'cite', 'del', 'ins',
+                                'q', 'sub', 'sup');
+
     /**
      * Object constructor
      *
@@ -88,28 +99,28 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
         @$this->_doc->loadHTML($htmlData);
 
         if ($this->_doc->encoding === null) {
-        	// Document encoding is not recognized
+            // Document encoding is not recognized
 
             /** @todo improve HTML vs HTML fragment recognition */
-            if (preg_match('/<html>/i', $htmlData, $matches, PREG_OFFSET_CAPTURE)) {
+            if (preg_match('/<html[^>]*>/i', $htmlData, $matches, PREG_OFFSET_CAPTURE)) {
                 // It's an HTML document
                 // Add additional HEAD section and recognize document
                 $htmlTagOffset = $matches[0][1] + strlen($matches[0][0]);
 
-        		@$this->_doc->loadHTML(iconv($defaultEncoding, 'UTF-8//IGNORE', substr($htmlData, 0, $htmlTagOffset))
+                @$this->_doc->loadHTML(iconv($defaultEncoding, 'UTF-8//IGNORE', substr($htmlData, 0, $htmlTagOffset))
                                      . '<head><META HTTP-EQUIV="Content-type" CONTENT="text/html; charset=UTF-8"/></head>'
                                      . iconv($defaultEncoding, 'UTF-8//IGNORE', substr($htmlData, $htmlTagOffset)));
 
                 // Remove additional HEAD section
                 $xpath = new DOMXPath($this->_doc);
                 $head  = $xpath->query('/html/head')->item(0);
-        		$head->parentNode->removeChild($head);
-        	} else {
-        		// It's an HTML fragment
-        		@$this->_doc->loadHTML('<html><head><META HTTP-EQUIV="Content-type" CONTENT="text/html; charset=UTF-8"/></head><body>'
-        		                     . iconv($defaultEncoding, 'UTF-8//IGNORE', $htmlData)
-        		                     . '</body></html>');
-        	}
+                $head->parentNode->removeChild($head);
+            } else {
+                // It's an HTML fragment
+                @$this->_doc->loadHTML('<html><head><META HTTP-EQUIV="Content-type" CONTENT="text/html; charset=UTF-8"/></head><body>'
+                                     . iconv($defaultEncoding, 'UTF-8//IGNORE', $htmlData)
+                                     . '</body></html>');
+            }
 
         }
         /** @todo Add correction of wrong HTML encoding recognition processing
@@ -155,6 +166,14 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
                 $this->_links[] = $href;
             }
         }
+        $linkNodes = $this->_doc->getElementsByTagName('area');
+        foreach ($linkNodes as $linkNode) {
+            if (($href = $linkNode->getAttribute('href')) != '' &&
+                (!self::$_excludeNoFollowLinks  ||  strtolower($linkNode->getAttribute('rel')) != 'nofollow' )
+               ) {
+                $this->_links[] = $href;
+            }
+        }
         $this->_links = array_unique($this->_links);
 
         $linkNodes = $xpath->query('/html/head/link');
@@ -197,8 +216,10 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
     private function _retrieveNodeText(DOMNode $node, &$text)
     {
         if ($node->nodeType == XML_TEXT_NODE) {
-            $text .= $node->nodeValue ;
-            $text .= ' ';
+            $text .= $node->nodeValue;
+            if(!in_array($node->parentNode->tagName, $this->_inlineTags)) {
+                $text .= ' ';
+            }
         } else if ($node->nodeType == XML_ELEMENT_NODE  &&  $node->nodeName != 'script') {
             foreach ($node->childNodes as $childNode) {
                 $this->_retrieveNodeText($childNode, $text);
@@ -264,6 +285,9 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
      */
     protected function _highlightTextNode(DOMText $node, $wordsToHighlight, $callback, $params)
     {
+        /** Zend_Search_Lucene_Analysis_Analyzer */
+        require_once 'Zend/Search/Lucene/Analysis/Analyzer.php';
+
         $analyzer = Zend_Search_Lucene_Analysis_Analyzer::getDefault();
         $analyzer->setInput($node->nodeValue, 'UTF-8');
 
@@ -301,16 +325,16 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
                                        . $highlightedWordNodeSetHtml
                                        . '</body></html>');
             if (!$success) {
-            	require_once 'Zend/Search/Lucene/Exception.php';
-            	throw new Zend_Search_Lucene_Exception("Error occured while loading highlighted text fragment: '$highlightedNodeHtml'.");
+                require_once 'Zend/Search/Lucene/Exception.php';
+                throw new Zend_Search_Lucene_Exception("Error occured while loading highlighted text fragment: '$highlightedWordNodeSetHtml'.");
             }
             $highlightedWordNodeSetXpath = new DOMXPath($highlightedWordNodeSetDomDocument);
             $highlightedWordNodeSet      = $highlightedWordNodeSetXpath->query('/html/body')->item(0)->childNodes;
 
             for ($count = 0; $count < $highlightedWordNodeSet->length; $count++) {
-            	$nodeToImport = $highlightedWordNodeSet->item($count);
-            	$node->parentNode->insertBefore($this->_doc->importNode($nodeToImport, true /* deep copy */),
-            	                                $matchedWordNode);
+                $nodeToImport = $highlightedWordNodeSet->item($count);
+                $node->parentNode->insertBefore($this->_doc->importNode($nodeToImport, true /* deep copy */),
+                                                $matchedWordNode);
             }
 
             $node->parentNode->removeChild($matchedWordNode);
@@ -372,7 +396,7 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
      */
     public function highlight($words, $colour = '#66ffff')
     {
-    	return $this->highlightExtended($words, array($this, 'applyColour'), array($colour));
+        return $this->highlightExtended($words, array($this, 'applyColour'), array($colour));
     }
 
 
@@ -389,6 +413,9 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
      */
     public function highlightExtended($words, $callback, $params = array())
     {
+        /** Zend_Search_Lucene_Analysis_Analyzer */
+        require_once 'Zend/Search/Lucene/Analysis/Analyzer.php';
+
         if (!is_array($words)) {
             $words = array($words);
         }
@@ -410,8 +437,8 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
         }
 
         if (!is_callable($callback)) {
-        	require_once 'Zend/Search/Lucene/Exception.php';
-        	throw new Zend_Search_Lucene_Exception('$viewHelper parameter mast be a View Helper name, View Helper object or callback.');
+            require_once 'Zend/Search/Lucene/Exception.php';
+            throw new Zend_Search_Lucene_Exception('$viewHelper parameter must be a View Helper name, View Helper object or callback.');
         }
 
         $xpath = new DOMXPath($this->_doc);
@@ -445,7 +472,7 @@ class Zend_Search_Lucene_Document_Html extends Zend_Search_Lucene_Document
 
         $outputFragments = array();
         for ($count = 0; $count < $bodyNodes->length; $count++) {
-        	$outputFragments[] = $this->_doc->saveXML($bodyNodes->item($count));
+            $outputFragments[] = $this->_doc->saveXML($bodyNodes->item($count));
         }
 
         return implode($outputFragments);
