@@ -15,9 +15,9 @@
  * @category   Zend
  * @package    Zend_Service
  * @subpackage Amazon_S3
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: S3.php 16971 2009-07-22 18:05:45Z mikaelkael $
+ * @version    $Id: S3.php 24593 2012-01-05 20:35:02Z matthew $
  */
 
 /**
@@ -36,7 +36,7 @@ require_once 'Zend/Crypt/Hmac.php';
  * @category   Zend
  * @package    Zend_Service
  * @subpackage Amazon_S3
- * @copyright  Copyright (c) 2005-2009 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  * @see        http://docs.amazonwebservices.com/AmazonS3/2006-03-01/
  */
@@ -157,14 +157,15 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
     public function createBucket($bucket, $location = null)
     {
         $this->_validBucketName($bucket);
-
+        $headers=array();
         if($location) {
             $data = '<CreateBucketConfiguration><LocationConstraint>'.$location.'</LocationConstraint></CreateBucketConfiguration>';
-        }
-        else {
+            $headers[self::S3_CONTENT_TYPE_HEADER]= 'text/plain';
+            $headers['Content-size']= strlen($data);
+        } else {
             $data = null;
         }
-        $response = $this->_makeRequest('PUT', $bucket, null, array(), $data);
+        $response = $this->_makeRequest('PUT', $bucket, null, $headers, $data);
 
         return ($response->getStatus() == 200);
     }
@@ -190,6 +191,7 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
      */
     public function isObjectAvailable($object)
     {
+        $object = $this->_fixupObjectName($object);
         $response = $this->_makeRequest('HEAD', $object);
 
         return ($response->getStatus() == 200);
@@ -272,23 +274,35 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
             return false;
         }
 
-        foreach ($objects as $object) {
-            $this->removeObject("$bucket/$object");
+        while (!empty($objects)) {
+            foreach ($objects as $object) {
+                $this->removeObject("$bucket/$object");
+            }
+            $params= array (
+                'marker' => $objects[count($objects)-1]
+            );
+            $objects = $this->getObjectsByBucket($bucket,$params);
         }
+        
         return true;
     }
 
     /**
      * List the objects in a bucket.
      *
-     * Provides the list of object keys that are contained in the bucket.
+     * Provides the list of object keys that are contained in the bucket.  Valid params include the following.
+     * prefix - Limits the response to keys which begin with the indicated prefix. You can use prefixes to separate a bucket into different sets of keys in a way similar to how a file system uses folders.
+     * marker - Indicates where in the bucket to begin listing. The list will only include keys that occur lexicographically after marker. This is convenient for pagination: To get the next page of results use the last key of the current page as the marker.
+     * max-keys - The maximum number of keys you'd like to see in the response body. The server might return fewer than this many keys, but will not return more.
+     * delimiter - Causes keys that contain the same string between the prefix and the first occurrence of the delimiter to be rolled up into a single result element in the CommonPrefixes collection. These rolled-up keys are not returned elsewhere in the response.
      *
      * @param  string $bucket
+     * @param array $params S3 GET Bucket Paramater
      * @return array|false
      */
-    public function getObjectsByBucket($bucket)
+    public function getObjectsByBucket($bucket, $params = array())
     {
-        $response = $this->_makeRequest('GET', $bucket);
+        $response = $this->_makeRequest('GET', $bucket, $params);
 
         if ($response->getStatus() != 200) {
             return false;
@@ -307,7 +321,52 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
 
         return $objects;
     }
+     /**
+     * List the objects and common prefixes in a bucket.
+     *
+     * Provides the list of object keys and common prefixes that are contained in the bucket.  Valid params include the following.
+     * prefix - Limits the response to keys which begin with the indicated prefix. You can use prefixes to separate a bucket into different sets of keys in a way similar to how a file system uses folders.
+     * marker - Indicates where in the bucket to begin listing. The list will only include keys that occur lexicographically after marker. This is convenient for pagination: To get the next page of results use the last key of the current page as the marker.
+     * max-keys - The maximum number of keys you'd like to see in the response body. The server might return fewer than this many keys, but will not return more.
+     * delimiter - Causes keys that contain the same string between the prefix and the first occurrence of the delimiter to be rolled up into a single result element in the CommonPrefixes collection. These rolled-up keys are not returned elsewhere in the response.
+     *
+     * @see ZF-11401
+     * @param  string $bucket
+     * @param array $params S3 GET Bucket Paramater
+     * @return array|false
+     */
+    public function getObjectsAndPrefixesByBucket($bucket, $params = array())
+    {
+        $response = $this->_makeRequest('GET', $bucket, $params);
 
+        if ($response->getStatus() != 200) {
+            return false;
+        }
+
+        $xml = new SimpleXMLElement($response->getBody());
+
+        $objects = array();
+        if (isset($xml->Contents)) {
+            foreach ($xml->Contents as $contents) {
+                foreach ($contents->Key as $object) {
+                    $objects[] = (string)$object;
+                }
+            }
+        }
+        $prefixes = array();
+        if (isset($xml->CommonPrefixes)) {
+            foreach ($xml->CommonPrefixes as $prefix) {
+                foreach ($prefix->Prefix as $object) {
+                    $prefixes[] = (string)$object;
+                }
+            }
+        }
+
+        return array(
+            'objects'  => $objects,
+            'prefixes' => $prefixes
+        );
+    }
     /**
      * Make sure the object name is valid
      *
@@ -352,11 +411,40 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
         return $response->getBody();
     }
 
+    /**
+     * Get an object using streaming
+     *
+     * Can use either provided filename for storage or create a temp file if none provided.
+     *
+     * @param  string $object Object path
+     * @param  string $streamfile File to write the stream to
+     * @param  bool   $paidobject This is "requestor pays" object
+     * @return Zend_Http_Response_Stream|false
+     */
+    public function getObjectStream($object, $streamfile = null, $paidobject=false)
+    {
+        $object = $this->_fixupObjectName($object);
+        self::getHttpClient()->setStream($streamfile?$streamfile:true);
+        if ($paidobject) {
+            $response = $this->_makeRequest('GET', $object, null, array(self::S3_REQUESTPAY_HEADER => 'requester'));
+        }
+        else {
+            $response = $this->_makeRequest('GET', $object);
+        }
+        self::getHttpClient()->setStream(null);
+
+        if ($response->getStatus() != 200 || !($response instanceof Zend_Http_Response_Stream)) {
+            return false;
+        }
+
+        return $response;
+    }
+
     /**
      * Upload an object by a PHP string
      *
      * @param  string $object Object name
-     * @param  string $data   Object data
+     * @param  string|resource $data   Object data (can be string or stream)
      * @param  array  $meta   Metadata
      * @return boolean
      */
@@ -365,7 +453,9 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
         $object = $this->_fixupObjectName($object);
         $headers = (is_array($meta)) ? $meta : array();
 
-        $headers['Content-MD5'] = base64_encode(md5($data, true));
+        if(!is_resource($data)) {
+            $headers['Content-MD5'] = base64_encode(md5($data, true));
+        }
         $headers['Expect'] = '100-continue';
 
         if (!isset($headers[self::S3_CONTENT_TYPE_HEADER])) {
@@ -379,7 +469,7 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
             // It is escaped by double quotes for some reason
             $etag = str_replace('"', '', $response->getHeader('Etag'));
 
-            if ($etag == md5($data)) {
+            if (is_resource($data) || $etag == md5($data)) {
                 return true;
             }
         }
@@ -417,6 +507,40 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
         return $this->putObject($object, $data, $meta);
     }
 
+    /**
+     * Put file to S3 as object, using streaming
+     *
+     * @param string $path   File name
+     * @param string $object Object name
+     * @param array  $meta   Metadata
+     * @return boolean
+     */
+    public function putFileStream($path, $object, $meta=null)
+    {
+        $data = @fopen($path, "rb");
+        if ($data === false) {
+            /**
+             * @see Zend_Service_Amazon_S3_Exception
+             */
+            require_once 'Zend/Service/Amazon/S3/Exception.php';
+            throw new Zend_Service_Amazon_S3_Exception("Cannot open file $path");
+        }
+
+        if (!is_array($meta)) {
+            $meta = array();
+        }
+
+        if (!isset($meta[self::S3_CONTENT_TYPE_HEADER])) {
+           $meta[self::S3_CONTENT_TYPE_HEADER] = self::getMimeType($path);
+        }
+
+        if(!isset($meta['Content-MD5'])) {
+            $meta['Content-MD5'] = base64_encode(md5_file($path, true));
+        }
+
+        return $this->putObject($object, $data, $meta);
+    }
+
     /**
      * Remove a given object
      *
@@ -432,14 +556,65 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
         return ($response->getStatus() == 204);
     }
 
+    /**
+     * Copy an object
+     *
+     * @param  string $sourceObject  Source object name
+     * @param  string $destObject    Destination object name
+     * @param  array  $meta          (OPTIONAL) Metadata to apply to desination object.
+     *                               Set to null to copy metadata from source object.
+     * @return boolean
+     */
+    public function copyObject($sourceObject, $destObject, $meta = null)
+    {
+        $sourceObject = $this->_fixupObjectName($sourceObject);
+        $destObject   = $this->_fixupObjectName($destObject);
+
+        $headers = (is_array($meta)) ? $meta : array();
+        $headers['x-amz-copy-source'] = $sourceObject;
+        $headers['x-amz-metadata-directive'] = $meta === null ? 'COPY' : 'REPLACE';
+
+        $response = $this->_makeRequest('PUT', $destObject, null, $headers);
+
+        if ($response->getStatus() == 200 && !stristr($response->getBody(), '<Error>')) {
+            return true;
+        }
+
+        return false;
+    }
+
+    /**
+     * Move an object
+     *
+     * Performs a copy to dest + verify + remove source
+     *
+     * @param string $sourceObject  Source object name
+     * @param string $destObject    Destination object name
+     * @param array  $meta          (OPTIONAL) Metadata to apply to destination object.
+     *                              Set to null to retain existing metadata.
+     */
+    public function moveObject($sourceObject, $destObject, $meta = null)
+    {
+        $sourceInfo = $this->getInfo($sourceObject);
+
+        $this->copyObject($sourceObject, $destObject, $meta);
+        $destInfo = $this->getInfo($destObject);
+
+        if ($sourceInfo['etag'] === $destInfo['etag']) {
+            return $this->removeObject($sourceObject);
+        } else {
+            return false;
+        }
+    }
+
     /**
      * Make a request to Amazon S3
      *
-     * @param  string $method
-     * @param  string $path
-     * @param  array  $params
-     * @param  array  $headers
-     * @param  string $data
+     * @param  string $method    Request method
+     * @param  string $path        Path to requested object
+     * @param  array  $params    Request parameters
+     * @param  array  $headers    HTTP headers
+     * @param  string|resource $data        Request data
      * @return Zend_Http_Response
      */
     public function _makeRequest($method, $path='', $params=null, $headers=array(), $data=null)
@@ -452,6 +627,14 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
 
         $headers['Date'] = gmdate(DATE_RFC1123, time());
 
+        if(is_resource($data) && $method != 'PUT') {
+            /**
+             * @see Zend_Service_Amazon_S3_Exception
+             */
+            require_once 'Zend/Service/Amazon/S3/Exception.php';
+            throw new Zend_Service_Amazon_S3_Exception("Only PUT request supports stream data");
+        }
+
         // build the end point out
         $parts = explode('/', $path, 2);
         $endpoint = clone($this->_endpoint);
@@ -460,7 +643,11 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
             $endpoint->setHost($parts[0].'.'.$endpoint->getHost());
         }
         if (!empty($parts[1])) {
-            $endpoint->setPath('/'.$parts[1]);
+            // ZF-10218, ZF-10122
+            $pathparts = explode('?',$parts[1]);
+            $endpath = $pathparts[0];
+            $endpoint->setPath('/'.$endpath);
+            
         }
         else {
             $endpoint->setPath('/');
@@ -468,21 +655,24 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
                 $path = $parts[0].'/';
             }
         }
-
         self::addSignature($method, $path, $headers);
 
         $client = self::getHttpClient();
 
-        $client->resetParameters();
+        $client->resetParameters(true);
+        $client->setUri($endpoint);
         $client->setAuth(false);
         // Work around buglet in HTTP client - it doesn't clean headers
         // Remove when ZHC is fixed
-        $client->setHeaders(array('Content-MD5' => null,
-                                  'Expect'      => null,
-                                  'Range'       => null,
-                                  'x-amz-acl'   => null));
-
-        $client->setUri($endpoint);
+        /*
+        $client->setHeaders(array('Content-MD5'              => null,
+                                  'Content-Encoding'         => null,
+                                  'Expect'                   => null,
+                                  'Range'                    => null,
+                                  'x-amz-acl'                => null,
+                                  'x-amz-copy-source'        => null,
+                                  'x-amz-metadata-directive' => null));
+        */
         $client->setHeaders($headers);
 
         if (is_array($params)) {
@@ -496,7 +686,7 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
                  $headers['Content-type'] = self::getMimeType($path);
              }
              $client->setRawData($data, $headers['Content-type']);
-         }
+         } 
          do {
             $retry = false;
 
@@ -587,6 +777,9 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
         else if (strpos($path, '?torrent') !== false) {
             $sig_str .= '?torrent';
         }
+        else if (strpos($path, '?versions') !== false) {
+            $sig_str .= '?versions';
+        }
 
         $signature = base64_encode(Zend_Crypt_Hmac::compute($this->_getSecretKey(), 'sha1', utf8_encode($sig_str), Zend_Crypt_Hmac::BINARY));
         $headers['Authorization'] = 'AWS '.$this->_getAccessKey().':'.$signature;
@@ -597,8 +790,6 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
     /**
      * Attempt to get the content-type of a file based on the extension
      *
-     * TODO: move this to Zend_Mime
-     *
      * @param  string $path
      * @return string
      */
@@ -611,7 +802,7 @@ class Zend_Service_Amazon_S3 extends Zend_Service_Amazon_Abstract
             return 'binary/octet-stream';
         }
 
-        switch ($ext) {
+        switch (strtolower($ext)) {
             case 'xls':
                 $content_type = 'application/excel';
                 break;
