@@ -15,9 +15,9 @@
  * @category   Zend
  * @package    Zend_Service
  * @subpackage Twitter
- * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
- * @version    $Id: Twitter.php 22629 2010-07-18 14:22:49Z padraic $
+ * @version    $Id: Twitter.php 25024 2012-07-30 15:08:15Z rob $
  */
 
 /**
@@ -39,7 +39,7 @@ require_once 'Zend/Oauth/Consumer.php';
  * @category   Zend
  * @package    Zend_Service
  * @subpackage Twitter
- * @copyright  Copyright (c) 2005-2010 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Service_Twitter extends Zend_Rest_Client
@@ -54,45 +54,45 @@ class Zend_Service_Twitter extends Zend_Rest_Client
      * This should be reviewed in the future...
      */
     const STATUS_MAX_CHARACTERS = 246;
-    
+
     /**
      * OAuth Endpoint
      */
     const OAUTH_BASE_URI = 'http://twitter.com/oauth';
-    
+
     /**
      * @var Zend_Http_CookieJar
      */
     protected $_cookieJar;
-    
+
     /**
      * Date format for 'since' strings
      *
      * @var string
      */
     protected $_dateFormat = 'D, d M Y H:i:s T';
-    
+
     /**
      * Username
      *
      * @var string
      */
     protected $_username;
-    
+
     /**
      * Current method type (for method proxying)
      *
      * @var string
      */
     protected $_methodType;
-    
+
     /**
      * Zend_Oauth Consumer
      *
      * @var Zend_Oauth_Consumer
      */
     protected $_oauthConsumer = null;
-    
+
     /**
      * Types of API methods
      *
@@ -107,7 +107,7 @@ class Zend_Service_Twitter extends Zend_Rest_Client
         'favorite',
         'block'
     );
-    
+
     /**
      * Options passed to constructor
      *
@@ -131,11 +131,15 @@ class Zend_Service_Twitter extends Zend_Rest_Client
     public function __construct($options = null, Zend_Oauth_Consumer $consumer = null)
     {
         $this->setUri('http://api.twitter.com');
-        if (!is_array($options)) $options = array();
-        $options['siteUrl'] = self::OAUTH_BASE_URI;
         if ($options instanceof Zend_Config) {
             $options = $options->toArray();
         }
+
+        if (!is_array($options)) {
+            $options = array();
+        }
+        $options['siteUrl'] = self::OAUTH_BASE_URI;
+
         $this->_options = $options;
         if (isset($options['username'])) {
             $this->setUsername($options['username']);
@@ -145,7 +149,7 @@ class Zend_Service_Twitter extends Zend_Rest_Client
             $this->setLocalHttpClient($options['accessToken']->getHttpClient($options));
         } else {
             $this->setLocalHttpClient(clone self::getHttpClient());
-            if (is_null($consumer)) {
+            if ($consumer === null) {
                 $this->_oauthConsumer = new Zend_Oauth_Consumer($options);
             } else {
                 $this->_oauthConsumer = $consumer;
@@ -166,7 +170,7 @@ class Zend_Service_Twitter extends Zend_Rest_Client
         $this->_localHttpClient->setHeaders('Accept-Charset', 'ISO-8859-1,utf-8');
         return $this;
     }
-    
+
     /**
      * Get the local HTTP client as distinct from the static HTTP client
      * inherited from Zend_Rest_Client
@@ -177,7 +181,7 @@ class Zend_Service_Twitter extends Zend_Rest_Client
     {
         return $this->_localHttpClient;
     }
-    
+
     /**
      * Checks for an authorised state
      *
@@ -358,6 +362,14 @@ class Zend_Service_Twitter extends Zend_Rest_Client
                 case 'page':
                     $_params['page'] = (int) $value;
                     break;
+                case 'max_id':
+                    $_params['max_id'] = $this->_validInteger($value);
+                    break;
+                case 'include_rts':
+                case 'trim_user':
+                case 'include_entities':
+                    $_params[strtolower($key)] = $value ? '1' : '0';
+                    break;                    
                 default:
                     break;
             }
@@ -376,8 +388,11 @@ class Zend_Service_Twitter extends Zend_Rest_Client
      * - page: return page X of results
      * - count: how many statuses to return
      * - max_id: returns only statuses with an ID less than or equal to the specified ID
-     * - user_id: specfies the ID of the user for whom to return the user_timeline
+     * - user_id: specifies the ID of the user for whom to return the user_timeline
      * - screen_name: specfies the screen name of the user for whom to return the user_timeline
+     * - include_rts: whether or not to return retweets
+     * - trim_user: whether to return just the user ID or a full user object; omit to return full object
+     * - include_entities: whether or not to return entities nodes with tweet metadata
      *
      * @throws Zend_Http_Client_Exception if HTTP request fails or times out
      * @return Zend_Rest_Client_Result
@@ -416,6 +431,11 @@ class Zend_Service_Twitter extends Zend_Rest_Client
                 case 'max_id':
                     $_params['max_id'] = $this->_validInteger($value);
                     break;
+                case 'include_rts':
+                case 'trim_user':
+                case 'include_entities':
+                    $_params[strtolower($key)] = $value ? '1' : '0';
+                    break;
                 default:
                     break;
             }
@@ -915,8 +935,7 @@ class Zend_Service_Twitter extends Zend_Rest_Client
 
     /**
      * Protected function to validate that the integer is valid or return a 0
-     * @param $int
-     * @throws Zend_Http_Client_Exception if HTTP request fails or times out
+     * @param mixed $int
      * @return integer
      */
     protected function _validInteger($int)
@@ -936,7 +955,7 @@ class Zend_Service_Twitter extends Zend_Rest_Client
      */
     protected function _validateScreenName($name)
     {
-        if (!preg_match('/^[a-zA-Z0-9_]{0,20}$/', $name)) {
+        if (!preg_match('/^[a-zA-Z0-9_]{0,15}$/', $name)) {
             require_once 'Zend/Service/Twitter/Exception.php';
             throw new Zend_Service_Twitter_Exception(
                 'Screen name, "' . $name
