@@ -14,8 +14,8 @@
  *
  * @category   Zend
  * @package    Zend_Translate
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
- * @version    $Id: Date.php 2498 2006-12-23 22:13:38Z thomas $
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
+ * @version    $Id: Tbx.php 24649 2012-02-26 03:37:54Z adamlundrigan $
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 
@@ -30,7 +30,7 @@ require_once 'Zend/Translate/Adapter.php';
 /**
  * @category   Zend
  * @package    Zend_Translate
- * @copyright  Copyright (c) 2005-2008 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright  Copyright (c) 2005-2012 Zend Technologies USA Inc. (http://www.zend.com)
  * @license    http://framework.zend.com/license/new-bsd     New BSD License
  */
 class Zend_Translate_Adapter_Tbx extends Zend_Translate_Adapter {
@@ -40,23 +40,8 @@ class Zend_Translate_Adapter_Tbx extends Zend_Translate_Adapter {
     private $_langset     = null;
     private $_termentry   = null;
     private $_content     = null;
-    private $_defined     = false;
     private $_term        = null;
-
-    /**
-     * Generates the tbx adapter
-     * This adapter reads with php's xml_parser
-     *
-     * @param  string              $data     Translation data
-     * @param  string|Zend_Locale  $locale   OPTIONAL Locale/Language to set, identical with locale identifier,
-     *                                       see Zend_Locale for more information
-     * @param  array               $options  OPTIONAL Options to set
-     */
-    public function __construct($data, $locale = null, array $options = array())
-    {
-        parent::__construct($data, $locale, $options);
-    }
-
+    private $_data        = array();
 
     /**
      * Load translation data (TBX file reader)
@@ -66,19 +51,11 @@ class Zend_Translate_Adapter_Tbx extends Zend_Translate_Adapter {
      *                            the source file
      * @param  array   $option    OPTIONAL Options to use
      * @throws Zend_Translation_Exception
+     * @return array
      */
     protected function _loadTranslationData($filename, $locale, array $options = array())
     {
-        $options = $options + $this->_options;
-
-        if ($options['clear']) {
-            $this->_translate = array();
-        }
-
-        if ((in_array('defined_language', $options)) and !empty($options['defined_language'])) {
-            $this->_defined = true;
-        }
-
+        $this->_data = array();
         if (!is_readable($filename)) {
             require_once 'Zend/Translate/Exception.php';
             throw new Zend_Translate_Exception('Translation file \'' . $filename . '\' is not readable.');
@@ -92,13 +69,16 @@ class Zend_Translate_Adapter_Tbx extends Zend_Translate_Adapter {
         xml_set_character_data_handler($this->_file, "_contentElement");
 
         if (!xml_parse($this->_file, file_get_contents($filename))) {
-            $ex = sprintf('XML error: %s at line %d',
+            $ex = sprintf('XML error: %s at line %d of file %s',
                           xml_error_string(xml_get_error_code($this->_file)),
-                          xml_get_current_line_number($this->_file));
+                          xml_get_current_line_number($this->_file),
+                          $filename);
             xml_parser_free($this->_file);
             require_once 'Zend/Translate/Exception.php';
             throw new Zend_Translate_Exception($ex);
         }
+
+        return $this->_data;
     }
 
     private function _startElement($file, $name, $attrib)
@@ -117,8 +97,8 @@ class Zend_Translate_Adapter_Tbx extends Zend_Translate_Adapter {
                 case 'langset':
                     if (isset($attrib['xml:lang']) === true) {
                         $this->_langset = $attrib['xml:lang'];
-                        if (isset($this->_translate[$this->_langset]) === false) {
-                            $this->_translate[$this->_langset] = array();
+                        if (isset($this->_data[$this->_langset]) === false) {
+                            $this->_data[$this->_langset] = array();
                         }
                     }
                     break;
@@ -146,8 +126,8 @@ class Zend_Translate_Adapter_Tbx extends Zend_Translate_Adapter {
                     if (empty($this->_termentry)) {
                         $this->_termentry = $this->_content;
                     }
-                    if (!empty($this->_content) or (isset($this->_translate[$this->_langset][$this->_termentry]) === false)) {
-                        $this->_translate[$this->_langset][$this->_termentry] = $this->_content;
+                    if (!empty($this->_content) or (isset($this->_data[$this->_langset][$this->_termentry]) === false)) {
+                        $this->_data[$this->_langset][$this->_termentry] = $this->_content;
                     }
                     break;
                 default:
@@ -167,8 +147,8 @@ class Zend_Translate_Adapter_Tbx extends Zend_Translate_Adapter {
     {
         $file = file_get_contents($filename, null, null, 0, 100);
         if (strpos($file, "encoding") !== false) {
-            $encoding = substr($file, strpos($file, "encoding") + 10);
-            $encoding = substr($encoding, 0, strpos($encoding, '"'));
+            $encoding = substr($file, strpos($file, "encoding") + 9);
+            $encoding = substr($encoding, 1, strpos($encoding, $encoding[0], 1) - 1);
             return $encoding;
         }
         return 'UTF-8';
